import httpx
from httpx_sse import aconnect_sse
from typing import Optional
from pydantic import BaseModel


class ReposAPI:
    def __init__(self, client: httpx.AsyncClient, token: str):
        self._client = client
        self.token = token

    async def add_repo(self, url: str):
        """Suggest repo(add repo if admin)"""
        payload = {
            "url": url,
            "token": self.token
        }
        response = await self._client.post("/repos/add", params=payload)
        response.raise_for_status()
        data = response.json()
        return data

    async def remove_repo(self, url: str, reason: None | str = None):
        """Remove repo(ADMIN ONLY)"""
        payload = {
            "url": url,
            "token": self.token,
        }
        if reason:
            payload["reason"] = reason
        
        response = await self._client.delete("/repos/remove", params=payload)
        response.raise_for_status()
        data = response.json()
        return data
    
    async def scan(self):
        """Rescan repos for new modules(ADMIN ONLY)"""
        payload = {
            "token": self.token
        }

        async with aconnect_sse(self._client, "GET", "/repos/scan", params=payload) as connection:
            print(connection.response.raise_for_status())
            data = []
            async for event in connection.aiter_sse():
                if event.data:
                    data.append(event.data)
            
            return data


import httpx
from typing import Optional

from .api.auth.api import AuthAPI
from .api.modules.api import ModulesAPI
from .api.checker.api import CheckerAPI
from .api.logger.api import LoggerAPI
from .api.repos.api import ReposAPI

class GoidaHetaAPIClient:
    API_PREFIX = "/api/v1"
    def __init__(self, base_url: str, token: str = ""):
        cleaned_url = base_url.rstrip('/')
        if not cleaned_url.endswith(self.API_PREFIX):
            self.base_url = cleaned_url + self.API_PREFIX
        else:
            self.base_url = cleaned_url
        self.client = httpx.AsyncClient(base_url=self.base_url)
        self.token = token
        if not self.token:
            raise ValueError("Token required!")
        self.auth = AuthAPI(self.client, token=self.token)
        self.checker = CheckerAPI(self.client, token=self.token)
        self.modules = ModulesAPI(self.client, token=self.token)
        self.logger = LoggerAPI(self.client, token=self.token)
        self.repos = ReposAPI(self.client,token=self.token)


import httpx
from typing import Optional

class AuthAPI:
    def __init__(self, client: httpx.AsyncClient, token: str):
        self._client = client
        self.token = token

    async def generate_token(self, tg_id: int):
        """(Re)Generate token for user"""
        params = {
            "token": self.token,
            "tg_id": tg_id,
        }
        response = await self._client.post("/auth/gen-token", params=params)
        response.raise_for_status()
        data = response.json()
        return data
    
    async def get_user(self, tg_id: int):
        """Returns user info"""
        params = {
            "token": self.token,
            "tg_id": tg_id,
        }
        response = await self._client.get("/auth/get-user", params=params)
        response.raise_for_status()
        data = response.json()
        return data
    
    async def get_me(self):
        """Returns your user info"""
        params = {
            "token": self.token
        }
        response = await self._client.get("/auth/get_me", params=params)
        response.raise_for_status()
        data = response.json()
        return data


import httpx
from typing import Optional

class ModulesAPI:
    def __init__(self, client: httpx.AsyncClient, token: str):
        self._client = client
        self.token = token
    
    async def get_rate(self, module_id: str):
        """Get rate for module with given id"""
        payload = {
            "module_id": module_id
        }
        response = await self._client.get("/modules/rate", params=payload)
        response.raise_for_status()
        data = response.json()
        return data
    
    async def search(self, query: str):
        """Search modules by query"""
        payload = {
            "query": query
        }
        response = await self._client.get("/modules/search", params=payload)
        response.raise_for_status()
        data = response.json()
        return data
    
    async def modules(self):
        """Get all modules(ADMIN ONLY)"""
        payload = {
            "token": self.token
        }
        response = await self._client.get("/modules/modules", params=payload)
        response.raise_for_status()
        data = response.json()
        return data
    
    async def module(self, module_id: int):
        """Get info about module by module_id"""
        response = await self._client.get(f"/modules/{module_id}")
        response.raise_for_status()
        data = response.json()
        return data

    async def rnd(self, count: int):
        """Get random module"""
        payload = {
            "count": count
        }
        response = await self._client.get("/modules/rnd", params=payload)
        response.raise_for_status()
        data = response.json()
        return data
    
    async def rate_module(self, module_id: int, rating: str):
        """Rate module(like or dislike)"""
        val = rating.lower()
        if val not in ["like", "dislike"]:
            raise ValueError(detail="Only like or dislike allowed")
        
        payload = {
            "module_id": module_id,
            "rating": val,
            "token": self.token
        }
        
        response = await self._client.post("/modules/rate", params=payload)
        response.raise_for_status()
        data = response.json()
        return data


import httpx
from typing import Optional

class CheckerAPI:
    def __init__(self, client: httpx.AsyncClient, token: str):
        self._client = client
        self.token = token

    async def verdict(self, source: str, hash: str, status: str, reason: Optional[str] = None):
        """Set verfict for something(ADMIN ONLY)"""
        payload = {
            "token": self.token,
            "source": source,
            "hash": hash,
            "status": status,
            "reason": reason,
        }
        response = await self._client.post("/checker/verdict", params=payload)
        response.raise_for_status()
        data = response.json()
        return data


import httpx
from typing import Optional
from httpx_sse import aconnect_sse

class LoggerAPI:
    def __init__(self, client: httpx.AsyncClient, token: str):
        self._client = client
        self.token = token

    async def get_logs(self):
        """Get actual logs(ADMIN ONLY)"""
        payload = {
            "token": self.token
        }
        response = await self._client.get("/logger/logs", params=payload)
        response.raise_for_status()
        data = response.text
        return data